/* GPS 0.1 - Gravitational Particle Simulator
 * Copyright (C) 2000 by Fabio Mancinelli <mancinel@univaq.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifdef HAVE_CONFIG_H
#include<config.h>
#endif

#include<about_dialog.h>

/******************************************************************************
  Callback function prototypes
 ******************************************************************************/

static gint cb_delete(GtkWidget *widget, GdkEvent *event, gpointer *data);
static void cb_ok(GtkWidget *widget, gpointer *data);

/******************************************************************************
  Exported functions
 ******************************************************************************/

ABOUT_DIALOG *about_dialog_new(void) {
        ABOUT_DIALOG *about_dialog;
        GtkWidget *window;
        GtkWidget *frame;
        GtkWidget *table;
        GtkWidget *label;
        GtkWidget *button;
        
        /* Allocate memory for the message dialog structure */
        about_dialog = (ABOUT_DIALOG *)g_malloc(sizeof(ABOUT_DIALOG));
        
        /* Create the dialog */
        window = gtk_dialog_new();
        about_dialog->window = window;
        gtk_window_set_title(GTK_WINDOW(window), "About");
        gtk_signal_connect(GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC(cb_delete), about_dialog);
        
        /* Create the frame */
        frame = gtk_frame_new("");
        gtk_widget_set_usize(frame, 480, 100);
        gtk_container_set_border_width(GTK_CONTAINER(frame), 5);
        
        table = gtk_table_new(4, 1, FALSE);
                
        label = gtk_label_new("GPS");
        gtk_widget_show(label);
        gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
        
        label = gtk_label_new("Gravitational Particle Simulator");
        gtk_widget_show(label);
        gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1, 2);

        label = gtk_label_new("(C) 2000 by Fabio Mancinelli");
        gtk_widget_show(label);
        gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 3, 4);

        gtk_widget_show(table);
        gtk_container_add(GTK_CONTAINER(frame), table);       
        
        gtk_widget_show(frame);
        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox), frame, TRUE, TRUE, 0);
        
        /* Create the buttons */
        button = gtk_button_new_with_label("Ok");
        gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(cb_ok), about_dialog);
        gtk_widget_show(button);
        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->action_area), button, TRUE, TRUE, 0);
        
        return about_dialog;
};

void about_dialog_show(ABOUT_DIALOG *about_dialog) {
        gtk_widget_show(about_dialog->window);
        
        gtk_grab_add(about_dialog->window);
        gtk_main();
        gtk_grab_remove(about_dialog->window);
        
        gtk_widget_destroy(about_dialog->window);
};      

ABOUT_DIALOG_DATA *about_dialog_get_data(ABOUT_DIALOG *about_dialog) {
        return &about_dialog->data;
};
        
void about_dialog_free(ABOUT_DIALOG *about_dialog) {
        g_free(about_dialog);
};

/******************************************************************************
  Callback functions
 ******************************************************************************/

static gint cb_delete(GtkWidget *widget, GdkEvent *event, gpointer *data) {
        /* Prevent the closing of the dialog using the close button */
        return TRUE;
};

static void cb_ok(GtkWidget *widget, gpointer *data) {
        ABOUT_DIALOG *about_dialog;
        
        about_dialog = (ABOUT_DIALOG *)data;
        about_dialog->data.result = ID_OK;
        
        if(gtk_main_level() != 1) gtk_main_quit();
};
        
