/* GPS 0.1 - Gravitational Particle Simulator
 * Copyright (C) 2000 by Fabio Mancinelli <mancinel@univaq.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifdef HAVE_CONFIG_H
#include<config.h>
#endif

#include<color_dialog.h>
#include<message_dialog.h>

/******************************************************************************
  Callback function prototypes
 ******************************************************************************/

static gint cb_delete(GtkWidget *widget, GdkEvent *event, gpointer *data);
static void cb_ok(GtkWidget *widget, gpointer *data);
static void cb_cancel(GtkWidget *widget, gpointer *data);

/******************************************************************************
  Exported functions
 ******************************************************************************/

COLOR_DIALOG *color_dialog_new(gchar *title) {
        COLOR_DIALOG *color_dialog;
        GtkWidget *window;
        
        /* Allocate memory for the message dialog structure */
        color_dialog = (COLOR_DIALOG *)g_malloc(sizeof(COLOR_DIALOG));
        
        /* Create the dialog */
        window = gtk_color_selection_dialog_new(title);
        color_dialog->window = window;
        gtk_signal_connect(GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC(cb_delete), color_dialog);
        gtk_signal_connect(GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(window)->ok_button), "clicked", GTK_SIGNAL_FUNC(cb_ok), color_dialog);
        gtk_signal_connect(GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(window)->cancel_button), "clicked", GTK_SIGNAL_FUNC(cb_cancel), color_dialog);
                                
        return color_dialog;
};

void color_dialog_show(COLOR_DIALOG *color_dialog) {
        gtk_widget_show(color_dialog->window);
        
        gtk_grab_add(color_dialog->window);
        gtk_main();
        gtk_grab_remove(color_dialog->window);
        
        gtk_widget_destroy(color_dialog->window);
};      

COLOR_DIALOG_DATA *color_dialog_get_data(COLOR_DIALOG *color_dialog) {
        return &color_dialog->data;
};
        
void color_dialog_free(COLOR_DIALOG *color_dialog) {
        g_free(color_dialog);
};

/******************************************************************************
  Callback functions
 ******************************************************************************/

static gint cb_delete(GtkWidget *widget, GdkEvent *event, gpointer *data) {
        /* Prevent the closing of the dialog using the close button */
        return TRUE;
};

static void cb_ok(GtkWidget *widget, gpointer *data) {
        COLOR_DIALOG *color_dialog;
        static gdouble color[3];
        
        color_dialog = (COLOR_DIALOG *)data;
        color_dialog->data.result = ID_OK;
        
        gtk_color_selection_get_color(GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG(color_dialog->window)->colorsel), color);
        color_dialog->data.color.red = (guint16)(color[0] * 65535.0);
        color_dialog->data.color.green = (guint16)(color[1] * 65535.0);
        color_dialog->data.color.blue = (guint16)(color[2] * 65535.0);
                
        if(gtk_main_level() != 1) gtk_main_quit();
};
        
static void cb_cancel(GtkWidget *widget, gpointer *data) {
        COLOR_DIALOG *color_dialog;        
        
        color_dialog = (COLOR_DIALOG *)data;
        color_dialog->data.result = ID_CANCEL;
                
        if(gtk_main_level() != 1) gtk_main_quit();
};

