/* GPS 0.1 - Gravitational Particle Simulator
 * Copyright (C) 2000 by Fabio Mancinelli <mancinel@univaq.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifdef HAVE_CONFIG_H
#include<config.h>
#endif

#include<file_dialog.h>
#include<message_dialog.h>

/******************************************************************************
  Callback function prototypes
 ******************************************************************************/

static gint cb_delete(GtkWidget *widget, GdkEvent *event, gpointer *data);
static void cb_ok(GtkWidget *widget, gpointer *data);
static void cb_cancel(GtkWidget *widget, gpointer *data);

/******************************************************************************
  Exported functions
 ******************************************************************************/

FILE_DIALOG *file_dialog_new(gchar *title, gchar *pattern) {
        FILE_DIALOG *file_dialog;
        GtkWidget *window;
        
        /* Allocate memory for the message dialog structure */
        file_dialog = (FILE_DIALOG *)g_malloc(sizeof(FILE_DIALOG));
        
        /* Create the dialog */
        window = gtk_file_selection_new(title);
        file_dialog->window = window;
        gtk_window_set_title(GTK_WINDOW(window), title);
        gtk_widget_hide(GTK_FILE_SELECTION(window)->button_area);
        gtk_file_selection_complete(GTK_FILE_SELECTION(window), pattern);
        gtk_signal_connect(GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC(cb_delete), file_dialog);
        gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(window)->ok_button), "clicked", GTK_SIGNAL_FUNC(cb_ok), file_dialog);
        gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(window)->cancel_button), "clicked", GTK_SIGNAL_FUNC(cb_cancel), file_dialog);
                                
        return file_dialog;
};

void file_dialog_show(FILE_DIALOG *file_dialog) {
        gtk_widget_show(file_dialog->window);
        
        gtk_grab_add(file_dialog->window);
        gtk_main();
        gtk_grab_remove(file_dialog->window);
        
        gtk_widget_destroy(file_dialog->window);
};      

FILE_DIALOG_DATA *file_dialog_get_data(FILE_DIALOG *file_dialog) {
        return &file_dialog->data;
};
        
void file_dialog_free(FILE_DIALOG *file_dialog) {
        g_free(file_dialog);
};

/******************************************************************************
  Callback functions
 ******************************************************************************/

static gint cb_delete(GtkWidget *widget, GdkEvent *event, gpointer *data) {
        /* Prevent the closing of the dialog using the close button */
        return TRUE;
};

static void cb_ok(GtkWidget *widget, gpointer *data) {
        FILE_DIALOG *file_dialog;
        
        file_dialog = (FILE_DIALOG *)data;
        file_dialog->data.result = ID_OK;
        strcpy(file_dialog->data.file_name, gtk_file_selection_get_filename(GTK_FILE_SELECTION(file_dialog->window)));
        
        if(gtk_main_level() != 1) gtk_main_quit();
};
        
static void cb_cancel(GtkWidget *widget, gpointer *data) {
        FILE_DIALOG *file_dialog;        
        
        file_dialog = (FILE_DIALOG *)data;
        file_dialog->data.result = ID_CANCEL;
                
        if(gtk_main_level() != 1) gtk_main_quit();
};

