/* GPS 0.1 - Gravitational Particle Simulator
 * Copyright (C) 2000 by Fabio Mancinelli <mancinel@univaq.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifndef __GPS_H
#define __GPS_H

#include<GL/gl.h>
#include<glib.h>
#include<vector.h>

#define PARTICLE_NAME_MAX_LENGTH        64
#define TRAIL_MAX_POINTS                64       
typedef struct {
        gchar name[PARTICLE_NAME_MAX_LENGTH];
        double mass;
        GLfloat color[4];
        
        VECTOR position;
        VECTOR velocity;
        VECTOR acceleration;
        
        double min_distance;
        
        VECTOR trail[TRAIL_MAX_POINTS];    
                
        VECTOR v0;
        VECTOR p0;
        
        VECTOR v_k0;
        VECTOR p_k0;
        
        VECTOR v_k1;
        VECTOR p_k1;
        
        VECTOR v_k2;
        VECTOR p_k2;
        
        VECTOR v_k3;
        VECTOR p_k3;
        
        VECTOR v_k4;
        VECTOR p_k4;        
} PARTICLE;
        
typedef struct {
        double g;
        double step_size;
        gboolean variable_step_size;
        double max_step_size;
        double separation;
        GList *particles_list;
        VECTOR camera;
        VECTOR target;
        double fov;
} GPS;

GPS *gps_new(void);
void gps_free(GPS *gps);
gint gps_load(gchar *file_name, GPS *gps);
gint gps_save(gchar *file_name, GPS *gps);
void gps_add_particle(PARTICLE *particle, GPS *gps);
void gps_delete_particle(PARTICLE *particle, GPS *gps);
void gps_update(GPS *gps);

#endif
