/* GPS 0.1 - Gravitational Particle Simulator
 * Copyright (C) 2000 by Fabio Mancinelli <mancinel@univaq.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifdef HAVE_CONFIG_H
#include<config.h>
#endif

#include<gps_gl.h>

/******************************************************************************
  Exported functions
 ******************************************************************************/

GLint gps_gl_xz_grid(double x_min, double x_max, double z_min, double z_max, double elevation, int x_lines, int z_lines) {
        static GLfloat white[4] = {0.8, 0.8, 0.8, 1.0};
        GLint gl_list;
        int i;
        
        gl_list = glGenLists(1);
        glNewList(gl_list, GL_COMPILE);
        
        glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, white); 
                
        glBegin(GL_LINES);
        
        for(i = 0; i < x_lines; i++) {                
                glVertex3d(x_min + (((x_max - x_min) / x_lines) * i), elevation, z_min);
                glVertex3d(x_min + (((x_max - x_min) / x_lines) * i), elevation, z_max);

        };
        for(i = 0; i < z_lines; i++) {
                glVertex3d(x_min, elevation, z_min + (((z_max - z_min) / z_lines) * i));
                glVertex3d(x_max, elevation, z_min + (((z_max - z_min) / z_lines) * i));                
        };
        glEnd();
        glEndList();
        
        return gl_list;
};

GLint gps_gl_axis(double axis_radius, double axis_length, double arrow_radius, double arrow_length) {
        GLint gl_list;
        GLUquadricObj *q;
        static GLfloat red[4] = {0.8, 0.1, 0.0, 1.0};
        static GLfloat green[4] = {0.0, 0.8, 0.0, 1.0};
        static GLfloat blue[4] = {0.0, 0.1, 0.8, 1.0};
        static GLfloat white[4] = {0.8, 0.8, 0.8, 1.0};
        
        
        gl_list = glGenLists(1);
        glNewList(gl_list, GL_COMPILE);
        
        glPushMatrix();
        glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, white); 
        glutSolidSphere(axis_radius * 2.0, 10, 10);
        glPopMatrix();

        q = gluNewQuadric();

        /* X - Axis */
        glPushMatrix();
        /*q = gluNewQuadric();*/
        glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, red); 
        gluQuadricDrawStyle(q, GLU_FILL);
        gluQuadricNormals(q, GLU_SMOOTH);
        glRotatef(90.0, 0.0, 1.0, 0.0);
        gluCylinder(q, axis_radius, axis_radius, axis_length, 15, 15);
        glPopMatrix();

        glPushMatrix();
        /*q = gluNewQuadric();*/
        gluQuadricDrawStyle(q, GLU_FILL);
        gluQuadricNormals(q, GLU_SMOOTH);
        glRotatef(90.0, 0.0, 1.0, 0.0);
        glTranslated(0.0, 0.0, axis_length);
        gluCylinder(q, arrow_radius, 0.0, arrow_length, 15, 15);
        glPopMatrix();

        /* Y - Axis */
        glPushMatrix();
        /*q = gluNewQuadric();*/
        glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, green); 
        gluQuadricDrawStyle(q, GLU_FILL);
        gluQuadricNormals(q, GLU_SMOOTH);
        glRotatef(-90.0, 1.0, 0.0, 0.0);
        gluCylinder(q, axis_radius, axis_radius, axis_length, 15, 15);
        glPopMatrix();

        glPushMatrix();
        /*q = gluNewQuadric();*/
        gluQuadricDrawStyle(q, GLU_FILL);
        gluQuadricNormals(q, GLU_SMOOTH);
        glRotatef(-90.0, 1.0, 0.0, 0.0);
        glTranslated(0.0, 0.0, axis_length);
        gluCylinder(q, arrow_radius, 0.0, arrow_length, 15, 15);
        glPopMatrix();


        /* Z - Axis */
        glPushMatrix();
        /*q = gluNewQuadric(); */
        glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, blue); 
        gluQuadricDrawStyle(q, GLU_FILL);
        gluQuadricNormals(q, GLU_SMOOTH);  
        gluCylinder(q, axis_radius, axis_radius, axis_length, 15, 15);
        glPopMatrix();

        glPushMatrix();
        /* q = gluNewQuadric(); */
        gluQuadricDrawStyle(q, GLU_FILL);
        gluQuadricNormals(q, GLU_SMOOTH);  
        glTranslated(0.0, 0.0, axis_length);
        gluCylinder(q, arrow_radius, 0.0, arrow_length, 15, 15);
        glPopMatrix();
                
        glEndList();
        
        return gl_list;
};

void gps_gl_draw_particle(PARTICLE *particle) {    
        int i;
        
        glPushMatrix();        
        glTranslated(particle->position.x, particle->position.y, particle->position.z);        
        glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, particle->color); 
        glutSolidSphere(0.2, 10, 10);
        glPopMatrix();
        
        glBegin(GL_LINES);
        glVertex3d(particle->position.x, particle->position.y, particle->position.z);
        glVertex3d(particle->trail[0].x, particle->trail[0].y, particle->trail[0].z);
        glEnd();
        
        for(i = 0; i < (TRAIL_MAX_POINTS - 1); i++) {                
                glBegin(GL_LINES);
                glVertex3d(particle->trail[i].x, particle->trail[i].y, particle->trail[i].z);
                glVertex3d(particle->trail[i + 1].x, particle->trail[i + 1].y, particle->trail[i + 1].z);
                glEnd();
        };
};

void gps_gl_draw_particle_bounding_box(PARTICLE *particle) {
        static GLfloat white[4] = {0.8, 0.8, 0.8, 1.0};
        
        glPushMatrix();        
        glTranslated(particle->position.x, particle->position.y, particle->position.z);        
        glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, white);         
        glutWireCube(0.5);
        glPopMatrix();
};        
        
