/* GPS 0.1 - Gravitational Particle Simulator
 * Copyright (C) 2000 by Fabio Mancinelli <mancinel@univaq.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifdef HAVE_CONFIG_H
#include<config.h>
#endif

#include<message_dialog.h>

/******************************************************************************
  Callback function prototypes
 ******************************************************************************/

static gint cb_delete(GtkWidget *widget, GdkEvent *event, gpointer *data);
static void cb_ok(GtkWidget *widget, gpointer *data);
static void cb_yes(GtkWidget *widget, gpointer *data);
static void cb_no(GtkWidget *widget, gpointer *data);
static void cb_cancel(GtkWidget *widget, gpointer *data);

/******************************************************************************
  Exported functions
 ******************************************************************************/

MESSAGE_DIALOG *message_dialog_new(gchar *title, gchar *message, gint flag) {
        MESSAGE_DIALOG *message_dialog;
        GtkWidget *window;
        GtkWidget *frame;
        GtkWidget *label;
        GtkWidget *button;
        
        /* Allocate memory for the message dialog structure */
        message_dialog = (MESSAGE_DIALOG *)g_malloc(sizeof(MESSAGE_DIALOG));
        
        /* Create the dialog */
        window = gtk_dialog_new();
        message_dialog->window = window;
        gtk_window_set_title(GTK_WINDOW(window), title);
        gtk_window_set_policy(GTK_WINDOW(window), FALSE, FALSE, FALSE);
        gtk_signal_connect(GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC(cb_delete), message_dialog);
        
        /* Create the frame */
        frame = gtk_frame_new("");
        gtk_widget_set_usize(frame, 250, 50);
        gtk_container_set_border_width(GTK_CONTAINER(frame), 5);
                
        label = gtk_label_new(message);
        gtk_widget_show(label);
        gtk_container_add(GTK_CONTAINER(frame), label);
        
        gtk_widget_show(frame);
        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox), frame, TRUE, TRUE, 0);
        
        /* Create the buttons */
        if(flag & OK_BUTTON) {
                button = gtk_button_new_with_label("Ok");
                gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(cb_ok), message_dialog);
                gtk_widget_show(button);
                gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->action_area), button, TRUE, TRUE, 0);
        };
        
        if(flag & YES_BUTTON) {
                button = gtk_button_new_with_label("Yes");
                gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(cb_yes), message_dialog);
                gtk_widget_show(button);
                gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->action_area), button, TRUE, TRUE, 0);
        };

        if(flag & NO_BUTTON) {
                button = gtk_button_new_with_label("No");
                gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(cb_no), message_dialog);
                gtk_widget_show(button);
                gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->action_area), button, TRUE, TRUE, 0);
        };

        if(flag & CANCEL_BUTTON) {
                button = gtk_button_new_with_label("Cancel");
                gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(cb_cancel), message_dialog);
                gtk_widget_show(button);
                gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->action_area), button, TRUE, TRUE, 0);
        };
                
        
        return message_dialog;
};

void message_dialog_show(MESSAGE_DIALOG *message_dialog) {
        gtk_widget_show(message_dialog->window);
        
        gtk_grab_add(message_dialog->window);
        gtk_main();
        gtk_grab_remove(message_dialog->window);
        
        gtk_widget_destroy(message_dialog->window);
};      

MESSAGE_DIALOG_DATA *message_dialog_get_data(MESSAGE_DIALOG *message_dialog) {
        return &message_dialog->data;
};
        
void message_dialog_free(MESSAGE_DIALOG *message_dialog) {
        g_free(message_dialog);
};

/******************************************************************************
  Callback functions
 ******************************************************************************/

static gint cb_delete(GtkWidget *widget, GdkEvent *event, gpointer *data) {
        /* Prevent the closing of the dialog using the close button */
        return TRUE;
};

static void cb_ok(GtkWidget *widget, gpointer *data) {
        MESSAGE_DIALOG *message_dialog;
        
        message_dialog = (MESSAGE_DIALOG *)data;
        message_dialog->data.result = ID_OK;
        
        if(gtk_main_level() != 1) gtk_main_quit();
};
        
static void cb_yes(GtkWidget *widget, gpointer *data) {
        MESSAGE_DIALOG *message_dialog;
        
        message_dialog = (MESSAGE_DIALOG *)data;
        message_dialog->data.result = ID_YES;
        
        if(gtk_main_level() != 1) gtk_main_quit();
};


static void cb_no(GtkWidget *widget, gpointer *data) {
        MESSAGE_DIALOG *message_dialog;
        
        message_dialog = (MESSAGE_DIALOG *)data;
        message_dialog->data.result = ID_NO;
        
        if(gtk_main_level() != 1) gtk_main_quit();
};

static void cb_cancel(GtkWidget *widget, gpointer *data) {
        MESSAGE_DIALOG *message_dialog;
        
        message_dialog = (MESSAGE_DIALOG *)data;
        message_dialog->data.result = ID_CANCEL;
        
        if(gtk_main_level() != 1) gtk_main_quit();
};

