/* GPS 0.1 - Gravitational Particle Simulator
 * Copyright (C) 2000 by Fabio Mancinelli <mancinel@univaq.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifdef HAVE_CONFIG_H
#include<config.h>
#endif

#include<stdio.h>
#include<stdlib.h>
#include<particle_dialog.h>
#include<message_dialog.h>
#include<color_dialog.h>

/******************************************************************************
  Static function prototypes
 ******************************************************************************/

static void color_area_set_color(GtkWidget *color_area, GdkColor *color);


/******************************************************************************
  Callback function prototypes
 ******************************************************************************/

static gint cb_delete(GtkWidget *widget, GdkEvent *event, gpointer *data);

static void cb_color_area_expose(GtkWidget *widget, GdkEventExpose *event, gpointer data);

static void cb_color(GtkWidget *widget, gpointer *data);
static void cb_ok(GtkWidget *widget, gpointer *data);
static void cb_cancel(GtkWidget *widget, gpointer *data);

/******************************************************************************
  Exported functions
 ******************************************************************************/

PARTICLE_DIALOG *particle_dialog_new(gchar *title, PARTICLE *particle) {
        PARTICLE_DIALOG *particle_dialog;
        GtkWidget *window;
        GtkWidget *frame;
        GtkWidget *table;
        GtkWidget *label;
        GtkWidget *entry;
        GtkWidget *color_area;
        GtkWidget *button;
        static gchar buffer[64];
        
        /* Allocate memory for the message dialog structure */
        particle_dialog = (PARTICLE_DIALOG *)g_malloc(sizeof(PARTICLE_DIALOG));
        
        /* Create the dialog */
        window = gtk_dialog_new();
        particle_dialog->window = window;
        
        /* Initialize the particle */
        if(particle != NULL) memcpy(&particle_dialog->data.particle, particle, sizeof(PARTICLE));         
        else {
                strcpy(particle_dialog->data.particle.name, "New particle");
                particle_dialog->data.particle.mass = 1.0;
                particle_dialog->data.particle.color[0] = 1.0;
                particle_dialog->data.particle.color[1] = 1.0;
                particle_dialog->data.particle.color[2] = 1.0;
                particle_dialog->data.particle.color[3] = 1.0;
                vector_set(&particle_dialog->data.particle.position, 0.0, 0.0, 0.0);
                vector_set(&particle_dialog->data.particle.velocity, 0.0, 1.0, 0.0);
        };                
        
        gtk_window_set_title(GTK_WINDOW(window), title);        
        gtk_container_set_border_width(GTK_CONTAINER(GTK_DIALOG(window)->vbox), 5);
        gtk_signal_connect(GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC(cb_delete), particle_dialog);

        /* Create the properties frame and its controls */
        frame = gtk_frame_new("Properties");
        
        table = gtk_table_new(3, 2, FALSE);
        gtk_container_set_border_width(GTK_CONTAINER(table), 10);
        gtk_table_set_col_spacing(GTK_TABLE(table), 0, 10);
        gtk_table_set_row_spacing(GTK_TABLE(table), 1, 10);
        
        label = gtk_label_new("Name :");
        gtk_widget_show(label);        
        gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
        
        entry = gtk_entry_new();
        particle_dialog->name = entry;
        gtk_entry_set_text(GTK_ENTRY(entry), particle_dialog->data.particle.name);
        gtk_widget_show(entry);
        gtk_table_attach_defaults(GTK_TABLE(table), entry, 1, 2, 0, 1);
        
        label = gtk_label_new("Mass :");
        gtk_widget_show(label);        
        gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1, 2);
        
        entry = gtk_entry_new();
        particle_dialog->mass = entry;        
        sprintf(buffer, "%f", particle_dialog->data.particle.mass);
        gtk_entry_set_text(GTK_ENTRY(entry), buffer);
        gtk_widget_show(entry);
        gtk_table_attach_defaults(GTK_TABLE(table), entry, 1, 2, 1, 2);
        
        button = gtk_button_new_with_label("Color");
        gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(cb_color), particle_dialog);
        gtk_widget_show(button);
        gtk_table_attach_defaults(GTK_TABLE(table), button, 0, 1, 2, 3);
        
        color_area = gtk_drawing_area_new();
        particle_dialog->color_area = color_area;
        particle_dialog->color.red = (guint16)(particle_dialog->data.particle.color[0] * 65535.0);
        particle_dialog->color.green = (guint16)(particle_dialog->data.particle.color[1] * 65535.0);;
        particle_dialog->color.blue = (guint16)(particle_dialog->data.particle.color[2] * 65535.0);
        gtk_signal_connect(GTK_OBJECT(color_area), "expose_event", GTK_SIGNAL_FUNC(cb_color_area_expose), particle_dialog);
        gtk_widget_show(color_area);
        gtk_table_attach_defaults(GTK_TABLE(table), color_area, 1, 2, 2, 3);
        
        gtk_widget_show(table);
        gtk_container_add(GTK_CONTAINER(frame), table);
        
        gtk_widget_show(frame);
        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox), frame, TRUE, TRUE, 0);                


        /* Create the position frame and its controls */
        frame = gtk_frame_new("Position");
        
        table = gtk_table_new(3, 2, FALSE);
        gtk_container_set_border_width(GTK_CONTAINER(table), 10);
        gtk_table_set_col_spacing(GTK_TABLE(table), 0, 10);
        
        label = gtk_label_new("X :");
        gtk_widget_show(label);        
        gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
        
        entry = gtk_entry_new();
        particle_dialog->position_x = entry;
        sprintf(buffer, "%f", particle_dialog->data.particle.position.x);
        gtk_entry_set_text(GTK_ENTRY(entry), buffer);        
        gtk_widget_show(entry);
        gtk_table_attach_defaults(GTK_TABLE(table), entry, 1, 2, 0, 1);
        
        label = gtk_label_new("Y :");
        gtk_widget_show(label);        
        gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1, 2);
        
        entry = gtk_entry_new();
        particle_dialog->position_y = entry;
        sprintf(buffer, "%f", particle_dialog->data.particle.position.y);
        gtk_entry_set_text(GTK_ENTRY(entry), buffer);                
        gtk_widget_show(entry);
        gtk_table_attach_defaults(GTK_TABLE(table), entry, 1, 2, 1, 2);
        
        label = gtk_label_new("Z :");
        gtk_widget_show(label);        
        gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 2, 3);
        
        entry = gtk_entry_new();
        particle_dialog->position_z = entry;
        sprintf(buffer, "%f", particle_dialog->data.particle.position.z);
        gtk_entry_set_text(GTK_ENTRY(entry), buffer);                
        gtk_widget_show(entry);
        gtk_table_attach_defaults(GTK_TABLE(table), entry, 1, 2, 2, 3);
                               
        gtk_widget_show(table);
        gtk_container_add(GTK_CONTAINER(frame), table);
        
        gtk_widget_show(frame);
        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox), frame, TRUE, TRUE, 0);                
        
        /* Create the velocity frame and its controls */
        frame = gtk_frame_new("Velocity");
        
        table = gtk_table_new(3, 2, FALSE);
        gtk_container_set_border_width(GTK_CONTAINER(table), 10);
        gtk_table_set_col_spacing(GTK_TABLE(table), 0, 10);
        
        label = gtk_label_new("X :");
        gtk_widget_show(label);        
        gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
        
        entry = gtk_entry_new();
        particle_dialog->velocity_x = entry;
        sprintf(buffer, "%f", particle_dialog->data.particle.velocity.x);
        gtk_entry_set_text(GTK_ENTRY(entry), buffer);                
        gtk_widget_show(entry);
        gtk_table_attach_defaults(GTK_TABLE(table), entry, 1, 2, 0, 1);
        
        label = gtk_label_new("Y :");
        gtk_widget_show(label);        
        gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1, 2);
        
        entry = gtk_entry_new();
        particle_dialog->velocity_y = entry;
        sprintf(buffer, "%f", particle_dialog->data.particle.velocity.y);
        gtk_entry_set_text(GTK_ENTRY(entry), buffer);                        
        gtk_widget_show(entry);
        gtk_table_attach_defaults(GTK_TABLE(table), entry, 1, 2, 1, 2);
        
        label = gtk_label_new("Z :");
        gtk_widget_show(label);        
        gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 2, 3);
        
        entry = gtk_entry_new();
        particle_dialog->velocity_z = entry;
        sprintf(buffer, "%f", particle_dialog->data.particle.velocity.z);
        gtk_entry_set_text(GTK_ENTRY(entry), buffer);                        
        gtk_widget_show(entry);
        gtk_table_attach_defaults(GTK_TABLE(table), entry, 1, 2, 2, 3);
        
        gtk_widget_show(table);
        gtk_container_add(GTK_CONTAINER(frame), table);
                
        gtk_widget_show(frame);
        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox), frame, TRUE, TRUE, 0);                
        
        
        /* Create the OK and Cancel buttons */
        button = gtk_button_new_with_label("OK");
        gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(cb_ok), particle_dialog);
        gtk_widget_show(button);
        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->action_area), button, TRUE, TRUE, 0);
        
        button = gtk_button_new_with_label("Cancel");
        gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(cb_cancel), particle_dialog);
        gtk_widget_show(button);
        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->action_area), button, TRUE, TRUE, 0);        
                                
        return particle_dialog;
};

void particle_dialog_show(PARTICLE_DIALOG *particle_dialog) {
        gtk_widget_show(particle_dialog->window);
        
        gtk_grab_add(particle_dialog->window);
        gtk_main();
        gtk_grab_remove(particle_dialog->window);
        
        gtk_widget_destroy(particle_dialog->window);
};      

PARTICLE_DIALOG_DATA *particle_dialog_get_data(PARTICLE_DIALOG *particle_dialog) {
        return &particle_dialog->data;
};
        
void particle_dialog_free(PARTICLE_DIALOG *particle_dialog) {
        g_free(particle_dialog);
};

/******************************************************************************
  Static functions
 ******************************************************************************/

static void color_area_set_color(GtkWidget *color_area, GdkColor *color) {
        GdkColormap *colormap;        
        
        colormap = gdk_window_get_colormap(color_area->window);              
        gdk_color_alloc (colormap, color);
        gdk_window_set_background (color_area->window, color);

        gdk_window_clear(color_area->window);                
};

/******************************************************************************
  Callback functions
 ******************************************************************************/

static gint cb_delete(GtkWidget *widget, GdkEvent *event, gpointer *data) {
        /* Prevent the closing of the dialog using the close button */
        return TRUE;
};

static void cb_color_area_expose(GtkWidget *widget, GdkEventExpose *event, gpointer data) {
        PARTICLE_DIALOG *particle_dialog;
        
        particle_dialog = (PARTICLE_DIALOG *)data;
        color_area_set_color(particle_dialog->color_area, &particle_dialog->color);
};

static void cb_color(GtkWidget *widget, gpointer *data) {
        PARTICLE_DIALOG *particle_dialog;
        COLOR_DIALOG *color_dialog;
        COLOR_DIALOG_DATA *color_dialog_data;
        
        particle_dialog = (PARTICLE_DIALOG *)data;
        
        color_dialog = color_dialog_new("Particle color");
        color_dialog_show(color_dialog);
        
        color_dialog_data = color_dialog_get_data(color_dialog);
        
        if(color_dialog_data->result == ID_OK) {
                particle_dialog->color.red = color_dialog_data->color.red;
                particle_dialog->color.green = color_dialog_data->color.green;
                particle_dialog->color.blue = color_dialog_data->color.blue;
                
                color_area_set_color(particle_dialog->color_area, &color_dialog_data->color);
        };
        
        color_dialog_free(color_dialog);
        
};

static void cb_ok(GtkWidget *widget, gpointer *data) {
        PARTICLE_DIALOG *particle_dialog;
        int i;
        
        particle_dialog = (PARTICLE_DIALOG *)data;
        particle_dialog->data.result = ID_OK;
        
        /* Create the new particle */        
        strcpy(particle_dialog->data.particle.name, gtk_entry_get_text(GTK_ENTRY(particle_dialog->name)));
        particle_dialog->data.particle.mass = atof(gtk_entry_get_text(GTK_ENTRY(particle_dialog->mass)));
        particle_dialog->data.particle.color[0] = ((GLfloat)particle_dialog->color.red) / 65535.0;
        particle_dialog->data.particle.color[1] = ((GLfloat)particle_dialog->color.green) / 65535.0;
        particle_dialog->data.particle.color[2] = ((GLfloat)particle_dialog->color.blue) / 65535.0;
        particle_dialog->data.particle.color[3] = 1.0;
        particle_dialog->data.particle.position.x = atof(gtk_entry_get_text(GTK_ENTRY(particle_dialog->position_x)));
        particle_dialog->data.particle.position.y = atof(gtk_entry_get_text(GTK_ENTRY(particle_dialog->position_y)));
        particle_dialog->data.particle.position.z = atof(gtk_entry_get_text(GTK_ENTRY(particle_dialog->position_z)));
        particle_dialog->data.particle.velocity.x = atof(gtk_entry_get_text(GTK_ENTRY(particle_dialog->velocity_x)));
        particle_dialog->data.particle.velocity.y = atof(gtk_entry_get_text(GTK_ENTRY(particle_dialog->velocity_y)));
        particle_dialog->data.particle.velocity.z = atof(gtk_entry_get_text(GTK_ENTRY(particle_dialog->velocity_z)));

        /* Initialize particle's trail */
        for(i = 0; i < TRAIL_MAX_POINTS; i++) {
                vector_copy(&particle_dialog->data.particle.trail[i], &particle_dialog->data.particle.position);
        };
        
        if(gtk_main_level() != 1) gtk_main_quit();
};
        
static void cb_cancel(GtkWidget *widget, gpointer *data) {
        PARTICLE_DIALOG *particle_dialog;        
        
        particle_dialog = (PARTICLE_DIALOG *)data;
        particle_dialog->data.result = ID_CANCEL;
                               
        if(gtk_main_level() != 1) gtk_main_quit();
};

