/* GPS 0.1 - Gravitational Particle Simulator
 * Copyright (C) 2000 by Fabio Mancinelli <mancinel@univaq.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifdef HAVE_CONFIG_H
#include<config.h>
#endif

#include<stdio.h>
#include<stdlib.h>
#include<particle_info_dialog.h>
#include<message_dialog.h>
#include<color_dialog.h>
#include<gps.h>


/******************************************************************************
  Callback function prototypes
 ******************************************************************************/

static gint cb_delete(GtkWidget *widget, GdkEvent *event, gpointer *data);
static void cb_ok(GtkWidget *widget, gpointer *data);

/******************************************************************************
  Exported functions
 ******************************************************************************/

PARTICLE_INFO_DIALOG *particle_info_dialog_new(gchar *title, PARTICLE *particle) {
        PARTICLE_INFO_DIALOG *particle_info_dialog;
        GtkWidget *window;
        GtkWidget *frame;
        GtkWidget *table;
        GtkWidget *label;
        GtkWidget *button;
        static gchar buffer[64];
        
        /* Allocate memory for the message dialog structure */
        particle_info_dialog = (PARTICLE_INFO_DIALOG *)g_malloc(sizeof(PARTICLE_INFO_DIALOG));
        
        /* Create the dialog */
        window = gtk_dialog_new();
        particle_info_dialog->window = window;                
        gtk_window_set_title(GTK_WINDOW(window), title);        
        gtk_container_set_border_width(GTK_CONTAINER(GTK_DIALOG(window)->vbox), 5);
        gtk_signal_connect(GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC(cb_delete), particle_info_dialog);

        /* Create the properties frame and its controls */
        frame = gtk_frame_new("Properties");
        
        table = gtk_table_new(2, 2, FALSE);
        gtk_container_set_border_width(GTK_CONTAINER(table), 10);
        
        label = gtk_label_new("Name :");
        gtk_widget_show(label);        
        gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
        
        label = gtk_label_new(particle->name);
        gtk_widget_show(label);
        gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, 0, 1);                
        
        label = gtk_label_new("Mass :");
        gtk_widget_show(label);        
        gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1, 2);
        
        sprintf(buffer, "%f", particle->mass);
        label = gtk_label_new(buffer);
        gtk_widget_show(label);
        gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, 1, 2);
                
        gtk_widget_show(table);
        gtk_container_add(GTK_CONTAINER(frame), table);
        
        gtk_widget_show(frame);
        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox), frame, TRUE, TRUE, 0);                


        /* Create the position frame and its controls */
        frame = gtk_frame_new("Position");
        
        table = gtk_table_new(3, 2, FALSE);
        gtk_container_set_border_width(GTK_CONTAINER(table), 10);
        gtk_table_set_col_spacing(GTK_TABLE(table), 0, 10);
        
        label = gtk_label_new("X :");
        gtk_widget_show(label);        
        gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
                
        sprintf(buffer, "%f", particle->position.x);
        label = gtk_label_new(buffer);        
        gtk_widget_show(label);
        gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, 0, 1);
        
        label = gtk_label_new("Y :");
        gtk_widget_show(label);        
        gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1, 2);
        
        sprintf(buffer, "%f", particle->position.y);
        label = gtk_label_new(buffer);        
        gtk_widget_show(label);
        gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, 1, 2);
        
        label = gtk_label_new("Z :");
        gtk_widget_show(label);        
        gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 2, 3);
        
        sprintf(buffer, "%f", particle->position.z);
        label = gtk_label_new(buffer);        
        gtk_widget_show(label);
        gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, 2, 3);

                                       
        gtk_widget_show(table);
        gtk_container_add(GTK_CONTAINER(frame), table);
        
        gtk_widget_show(frame);
        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox), frame, TRUE, TRUE, 0);                
        
        /* Create the velocity frame and its controls */
        frame = gtk_frame_new("Velocity");
        
        table = gtk_table_new(3, 2, FALSE);
        gtk_container_set_border_width(GTK_CONTAINER(table), 10);
        gtk_table_set_col_spacing(GTK_TABLE(table), 0, 10);
        
        label = gtk_label_new("X :");
        gtk_widget_show(label);        
        gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
                
        sprintf(buffer, "%f", particle->velocity.x);
        label = gtk_label_new(buffer);        
        gtk_widget_show(label);
        gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, 0, 1);
        
        label = gtk_label_new("Y :");
        gtk_widget_show(label);        
        gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1, 2);
        
        sprintf(buffer, "%f", particle->velocity.y);
        label = gtk_label_new(buffer);        
        gtk_widget_show(label);
        gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, 1, 2);
        
        label = gtk_label_new("Z :");
        gtk_widget_show(label);        
        gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 2, 3);
        
        sprintf(buffer, "%f", particle->velocity.z);
        label = gtk_label_new(buffer);        
        gtk_widget_show(label);
        gtk_table_attach_defaults(GTK_TABLE(table), label, 1, 2, 2, 3);

        
        gtk_widget_show(table);
        gtk_container_add(GTK_CONTAINER(frame), table);
                
        gtk_widget_show(frame);
        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox), frame, TRUE, TRUE, 0);                
        
        
        /* Create the OK and Cancel buttons */
        button = gtk_button_new_with_label("OK");
        gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(cb_ok), particle_info_dialog);
        gtk_widget_show(button);
        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->action_area), button, TRUE, TRUE, 0);
                                        
        return particle_info_dialog;
};

void particle_info_dialog_show(PARTICLE_INFO_DIALOG *particle_info_dialog) {
        gtk_widget_show(particle_info_dialog->window);
        
        gtk_grab_add(particle_info_dialog->window);
        gtk_main();
        gtk_grab_remove(particle_info_dialog->window);
        
        gtk_widget_destroy(particle_info_dialog->window);
};      

PARTICLE_INFO_DIALOG_DATA *particle_info_dialog_get_data(PARTICLE_INFO_DIALOG *particle_info_dialog) {
        return &particle_info_dialog->data;
};
        
void particle_info_dialog_free(PARTICLE_INFO_DIALOG *particle_info_dialog) {
        g_free(particle_info_dialog);
};


/******************************************************************************
  Callback functions
 ******************************************************************************/

static gint cb_delete(GtkWidget *widget, GdkEvent *event, gpointer *data) {
        /* Prevent the closing of the dialog using the close button */
        return TRUE;
};



static void cb_ok(GtkWidget *widget, gpointer *data) {
        PARTICLE_INFO_DIALOG *particle_info_dialog;
        
        particle_info_dialog = (PARTICLE_INFO_DIALOG *)data;
        particle_info_dialog->data.result = ID_OK;
                
        if(gtk_main_level() != 1) gtk_main_quit();
};
