/* GPS 0.1 - Gravitational Particle Simulator
 * Copyright (C) 2000 by Fabio Mancinelli <mancinel@univaq.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifdef HAVE_CONFIG_H
#include<config.h>
#endif

#include<stdio.h>
#include<stdlib.h>
#include<settings_dialog.h>

/******************************************************************************
  Callback function prototypes
 ******************************************************************************/

static gint cb_delete(GtkWidget *widget, GdkEvent *event, gpointer *data);
static void cb_ok(GtkWidget *widget, gpointer *data);
static void cb_cancel(GtkWidget *widget, gpointer *data);

/******************************************************************************
  Exported functions
 ******************************************************************************/

SETTINGS_DIALOG *settings_dialog_new(GPS *gps) {
        SETTINGS_DIALOG *settings_dialog;
        GtkWidget *window;
        GtkWidget *frame;
        GtkWidget *table;
        GtkWidget *label;
        GtkWidget *entry;
        GtkWidget *check_button;
        GtkWidget *button;
        static gchar buffer[64];
        
        /* Allocate memory for the message dialog structure */
        settings_dialog = (SETTINGS_DIALOG *)g_malloc(sizeof(SETTINGS_DIALOG));
        
        /* Create the Settings dialog */
        window = gtk_dialog_new();
        settings_dialog->window = window;
        gtk_window_set_title(GTK_WINDOW(window), "Settings");        
        gtk_container_set_border_width(GTK_CONTAINER(GTK_DIALOG(window)->vbox), 5);
        gtk_signal_connect(GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC(cb_delete), settings_dialog);

        /* Create the properties frame and its controls */
        frame = gtk_frame_new("Parameters");
        
        table = gtk_table_new(5, 2, FALSE);
        gtk_container_set_border_width(GTK_CONTAINER(table), 10);
        
        label = gtk_label_new("G :");
        gtk_widget_show(label);        
        gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
        
        entry = gtk_entry_new();
        settings_dialog->g = entry;
        sprintf(buffer, "%f", gps->g);
        gtk_entry_set_text(GTK_ENTRY(entry), buffer);        
        gtk_widget_show(entry);
        gtk_table_attach_defaults(GTK_TABLE(table), entry, 1, 2, 0, 1);
        
        label = gtk_label_new("Step size :");
        gtk_widget_show(label);        
        gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1, 2);
        
        entry = gtk_entry_new();
        settings_dialog->step_size = entry;
        sprintf(buffer, "%f", gps->step_size);
        gtk_entry_set_text(GTK_ENTRY(entry), buffer);                
        gtk_widget_show(entry);
        gtk_table_attach_defaults(GTK_TABLE(table), entry, 1, 2, 1, 2);
                
        check_button = gtk_check_button_new_with_label("Variable step size");
        settings_dialog->variable_step_size = check_button;        
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button), gps->variable_step_size);        
        gtk_widget_show(check_button);
        gtk_table_attach_defaults(GTK_TABLE(table), check_button, 1, 2, 2, 3);        
        
        label = gtk_label_new("Max step size :");
        gtk_widget_show(label);        
        gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 3, 4);
        
        entry = gtk_entry_new();
        settings_dialog->max_step_size = entry;
        sprintf(buffer, "%f", gps->max_step_size);
        gtk_entry_set_text(GTK_ENTRY(entry), buffer);                
        gtk_widget_show(entry);
        gtk_table_attach_defaults(GTK_TABLE(table), entry, 1, 2, 3, 4);
        
        label = gtk_label_new("% Separation :");
        gtk_widget_show(label);        
        gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 4, 5);
        
        entry = gtk_entry_new();
        settings_dialog->separation = entry;
        sprintf(buffer, "%f", gps->separation);
        gtk_entry_set_text(GTK_ENTRY(entry), buffer);                
        gtk_widget_show(entry);
        gtk_table_attach_defaults(GTK_TABLE(table), entry, 1, 2, 4, 5);
        
        gtk_widget_show(table);
        gtk_container_add(GTK_CONTAINER(frame), table);
        
        gtk_widget_show(frame);
        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox), frame, TRUE, TRUE, 0);                

        
        /* Create the buttons */
        button = gtk_button_new_with_label("Ok");
        gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(cb_ok), settings_dialog);
        gtk_widget_show(button);
        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->action_area), button, TRUE, TRUE, 0);
        
        button = gtk_button_new_with_label("Cancel");
        gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(cb_cancel), settings_dialog);
        gtk_widget_show(button);
        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->action_area), button, TRUE, TRUE, 0);
                
        
        return settings_dialog;
};

void settings_dialog_show(SETTINGS_DIALOG *settings_dialog) {
        gtk_widget_show(settings_dialog->window);
        
        gtk_grab_add(settings_dialog->window);
        gtk_main();
        gtk_grab_remove(settings_dialog->window);
        
        gtk_widget_destroy(settings_dialog->window);
};      

SETTINGS_DIALOG_DATA *settings_dialog_get_data(SETTINGS_DIALOG *settings_dialog) {
        return &settings_dialog->data;
};
        
void settings_dialog_free(SETTINGS_DIALOG *settings_dialog) {
        g_free(settings_dialog);
};

/******************************************************************************
  Callback functions
 ******************************************************************************/

static gint cb_delete(GtkWidget *widget, GdkEvent *event, gpointer *data) {
        /* Prevent the closing of the dialog using the close button */
        return TRUE;
};

static void cb_ok(GtkWidget *widget, gpointer *data) {
        SETTINGS_DIALOG *settings_dialog;
        
        settings_dialog = (SETTINGS_DIALOG *)data;
        settings_dialog->data.result = ID_OK;
        
        settings_dialog->data.g = atof(gtk_entry_get_text(GTK_ENTRY(settings_dialog->g)));
        settings_dialog->data.step_size = atof(gtk_entry_get_text(GTK_ENTRY(settings_dialog->step_size)));
        settings_dialog->data.variable_step_size = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(settings_dialog->variable_step_size));
        settings_dialog->data.max_step_size = atof(gtk_entry_get_text(GTK_ENTRY(settings_dialog->max_step_size)));
        settings_dialog->data.separation = atof(gtk_entry_get_text(GTK_ENTRY(settings_dialog->separation)));
        
        if(gtk_main_level() != 1) gtk_main_quit();
};
        
static void cb_cancel(GtkWidget *widget, gpointer *data) {
        SETTINGS_DIALOG *settings_dialog;
        
        settings_dialog = (SETTINGS_DIALOG *)data;
        settings_dialog->data.result = ID_CANCEL;
        
        if(gtk_main_level() != 1) gtk_main_quit();
};

