/* GPS 0.1 - Gravitational Particle Simulator
 * Copyright (C) 2000 by Fabio Mancinelli <mancinel@univaq.it>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#ifdef HAVE_CONFIG_H
#include<config.h>
#endif

#include<stdio.h>
#include<stdlib.h>
#include<vector_dialog.h>
#include<message_dialog.h>
#include<color_dialog.h>


/******************************************************************************
  Callback function prototypes
 ******************************************************************************/

static gint cb_delete(GtkWidget *widget, GdkEvent *event, gpointer *data);
static void cb_ok(GtkWidget *widget, gpointer *data);
static void cb_cancel(GtkWidget *widget, gpointer *data);

/******************************************************************************
  Exported functions
 ******************************************************************************/

VECTOR_DIALOG *vector_dialog_new(gchar *title, VECTOR *v) {
        VECTOR_DIALOG *vector_dialog;
        GtkWidget *window;
        GtkWidget *frame;
        GtkWidget *table;
        GtkWidget *label;
        GtkWidget *entry;
        GtkWidget *button;
        static gchar buffer[64];
        
        /* Allocate memory for the message dialog structure */
        vector_dialog = (VECTOR_DIALOG *)g_malloc(sizeof(VECTOR_DIALOG));
        
        /* Create the dialog */
        window = gtk_dialog_new();
        vector_dialog->window = window;
        
        /* Initialize the particle */
        if(v != NULL) memcpy(&vector_dialog->data.v, v, sizeof(VECTOR));         
        else {
                vector_set(&vector_dialog->data.v, 0.0, 0.0, 0.0);
        };                
        
        gtk_window_set_title(GTK_WINDOW(window), title);        
        gtk_container_set_border_width(GTK_CONTAINER(GTK_DIALOG(window)->vbox), 5);
        gtk_signal_connect(GTK_OBJECT(window), "delete_event", GTK_SIGNAL_FUNC(cb_delete), vector_dialog);


        /* Create the position frame and its controls */
        frame = gtk_frame_new("");
        
        table = gtk_table_new(3, 2, FALSE);
        gtk_container_set_border_width(GTK_CONTAINER(table), 10);
        gtk_table_set_col_spacing(GTK_TABLE(table), 0, 10);
        
        label = gtk_label_new("X :");
        gtk_widget_show(label);        
        gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
        
        entry = gtk_entry_new();
        vector_dialog->x = entry;
        sprintf(buffer, "%f", vector_dialog->data.v.x);
        gtk_entry_set_text(GTK_ENTRY(entry), buffer);        
        gtk_widget_show(entry);
        gtk_table_attach_defaults(GTK_TABLE(table), entry, 1, 2, 0, 1);
        
        label = gtk_label_new("Y :");
        gtk_widget_show(label);        
        gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1, 2);
        
        entry = gtk_entry_new();
        vector_dialog->y = entry;
        sprintf(buffer, "%f", vector_dialog->data.v.y);
        gtk_entry_set_text(GTK_ENTRY(entry), buffer);                
        gtk_widget_show(entry);
        gtk_table_attach_defaults(GTK_TABLE(table), entry, 1, 2, 1, 2);
        
        label = gtk_label_new("Z :");
        gtk_widget_show(label);        
        gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 2, 3);
        
        entry = gtk_entry_new();
        vector_dialog->z = entry;
        sprintf(buffer, "%f", vector_dialog->data.v.z);
        gtk_entry_set_text(GTK_ENTRY(entry), buffer);                
        gtk_widget_show(entry);
        gtk_table_attach_defaults(GTK_TABLE(table), entry, 1, 2, 2, 3);
                               
        gtk_widget_show(table);
        gtk_container_add(GTK_CONTAINER(frame), table);
        
        gtk_widget_show(frame);
        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox), frame, TRUE, TRUE, 0);                
                
        
        /* Create the OK and Cancel buttons */
        button = gtk_button_new_with_label("OK");
        gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(cb_ok), vector_dialog);
        gtk_widget_show(button);
        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->action_area), button, TRUE, TRUE, 0);
        
        button = gtk_button_new_with_label("Cancel");
        gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(cb_cancel), vector_dialog);
        gtk_widget_show(button);
        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->action_area), button, TRUE, TRUE, 0);        
                                
        return vector_dialog;
};

void vector_dialog_show(VECTOR_DIALOG *vector_dialog) {
        gtk_widget_show(vector_dialog->window);
        
        gtk_grab_add(vector_dialog->window);
        gtk_main();
        gtk_grab_remove(vector_dialog->window);
        
        gtk_widget_destroy(vector_dialog->window);
};      

VECTOR_DIALOG_DATA *vector_dialog_get_data(VECTOR_DIALOG *vector_dialog) {
        return &vector_dialog->data;
};
        
void vector_dialog_free(VECTOR_DIALOG *vector_dialog) {
        g_free(vector_dialog);
};


/******************************************************************************
  Callback functions
 ******************************************************************************/

static gint cb_delete(GtkWidget *widget, GdkEvent *event, gpointer *data) {
        /* Prevent the closing of the dialog using the close button */
        return TRUE;
};


static void cb_ok(GtkWidget *widget, gpointer *data) {
        VECTOR_DIALOG *vector_dialog;
        
        vector_dialog = (VECTOR_DIALOG *)data;
        vector_dialog->data.result = ID_OK;
        
        vector_dialog->data.v.x = atof(gtk_entry_get_text(GTK_ENTRY(vector_dialog->x)));
        vector_dialog->data.v.y = atof(gtk_entry_get_text(GTK_ENTRY(vector_dialog->y)));
        vector_dialog->data.v.z = atof(gtk_entry_get_text(GTK_ENTRY(vector_dialog->z)));
        
        if(gtk_main_level() != 1) gtk_main_quit();
};
        
static void cb_cancel(GtkWidget *widget, gpointer *data) {
        VECTOR_DIALOG *vector_dialog;        
        
        vector_dialog = (VECTOR_DIALOG *)data;
        vector_dialog->data.result = ID_CANCEL;
                               
        if(gtk_main_level() != 1) gtk_main_quit();
};

